import datetime
from iotgate.core.model.managers import CommunicatorManager
from iotgate.core.model.nodes import NodeComponentState
from iotgate.core.model.adaptors import Adaptor
from iotgate.core.main import DBManager

__author__ = "diego"
__copyright__ = "Copyright 2016, Argotec SL"
__license__ = "?"
__version__ = "0.1.4-tfg"
__maintainer__ = "Diego Fidalgo"
__email__ = "dfidalgo@argotec.es"
__status__ = "Development"


class Thermostat(Adaptor):
    def __init__(self, id, communicators):
        Adaptor.__init__(self, id, Thermostat, communicators)

    def configure(self):
        server = self.communicators[0]
        server.add_callback("server_request", self.listener)

    def listener(self, address, client):
        thermostat = self.managed_nodes[address['device_id']]
        if thermostat:
            message = client.receive()
            rssi = message['rssi']
            payload = message['message']

            collected_data = {
                "humidity": payload.split(':')[1].split(',')[0],
                "temperature": payload.split(':')[1].split(',')[1],
                "comfort_temperature": payload.split(':')[1].split(',')[-1],
                "relay": 'True' if payload.split(':')[1].split(',')[2].split('-')[1] == '1' else 'False',
                "rssi": rssi
            }
            thermostat.update_state(collected_data)
        client.close()

    def send_action(self, actuator, action, params):
        payload = action.get_command() % params
        client_config = self.communicators[0].get_configuration()
        client_config.update({'device_id': actuator.get_node().get_network_id()})
        client = CommunicatorManager.create_communicator('ieee802154.IEEE802154Client', client_config)
        client.send(payload)
        client.close()

        return NodeComponentState(str(params['temperature']))
