import datetime
from iotgate.core.model.adaptors import Adaptor
from iotgate.core.model.managers import CommunicatorManager
from iotgate.core.model.nodes import NodeComponentState


__author__ = "diego"
__copyright__ = "Copyright 2016, Argotec SL"
__license__ = "?"
__version__ = "0.1.2"
__maintainer__ = "Diego Fidalgo"
__email__ = "dfidalgo@argotec.es"
__status__ = "Development"


class MCP4728(Adaptor):
    __mapper_args__ = {'polymorphic_identity': 'argotec.netbows.MCP4728'}

    registers = {
        'a': 0x58,
        'b': 0x5A,
        'c': 0x5C,
        'd': 0x5E
    }

    def __init__(self, id, communicators):
        Adaptor.__init__(self, id, MCP4728, communicators)
        self.client = CommunicatorManager.create_communicator('i2c.I2CClient', self.configuration)

    def configure(self):
        self.client.connect()

    def set_vdd_to_channel(self, board_id, value):
        original_value = value
        value = float(value)
        channel = board_id
        value = int(value * 1000)

        value = int(value / 2.96)
        if value > 4095:
            value = 4095

        values_to_write = [(value >> 8) & 0xFF,
                           value & 0xFF]
        message = {
            'register': self.registers[channel],
            'values': values_to_write,
        }
        self.get_managed_node('mpc4728').update_values({channel: original_value})
        self.client.send(message)

        return str(original_value)

    def send_action(self, actuator, action, **params):
        self.set_vdd_to_channel(**params)
        return NodeComponentState(received_at=datetime.datetime.utcnow(),
                                  observad_at=datetime.datetime.utcnow(),
                                  value=str(params))

    def __del__(self):
        self.client.close()

