import datetime

from iotgate.communicators.buses.i2c import I2C
from iotgate.core.model.communicators import Client

__author__ = "diego"
__copyright__ = "Copyright 2016, Argotec SL"
__license__ = "?"
__version__ = "0.1.2"
__maintainer__ = "Diego Fidalgo"
__email__ = "dfidalgo@argotec.es"
__status__ = "Development"


class I2CClient(Client):
    __mapper_args__ = {'polymorphic_identity': 'I2CClient'}

    def __init__(self, id, configuration):
        Client.__init__(self, id, 'i2c.I2CClient', configuration)
        self.port = None
        self.address = None
        self.connector = None
        self.register = None
        self.length = None
        self.configuration = None

    def configure(self):
        if "port" in self.configuration:
            self.port = int(self.configuration["port"])
        self.address = int(self.configuration["address"], 16)
        if "register" in self.configuration:
            self.register = int(self.configuration["register"])
            self.length = int(self.configuration["length"])

    def set_configuration(self, configuration):
        self.configuration = configuration

    def connect(self):
        if self.port:
            self.connector = I2C(address=self.address, bus_num=self.port)
            self.trigger_event("client_connect", communicator=self, configuration=self.configuration)
        else:
            self.connector = I2C(address=self.address)
            self.trigger_event("client_connect", communicator=self, configuration=self.configuration)

    def send(self, message):
        register = message["register"]
        values_to_write = message["values"]
        self.connector.write_list(reg=register, list=values_to_write)
        self.trigger_event("client_send", communicator=self, frame=message)

    def receive(self):
        response = self.connector.read_list(self.register, self.length)
        self.trigger_event("client_rcv", communicator=self, frame=response)

        return response

    def close(self):
        self.connector.close()
