from iotgate.core.model.communicators import Server
from iotgate.core.model.communicators import Client

from iotgate.communicators.drivers.microchip.MRF24J40M import MRF24J40MRadio

__author__ = "diego"
__copyright__ = "Copyright 2016, Argotec SL"
__license__ = "?"
__version__ = "0.1.4-tfg"
__maintainer__ = "Diego Fidalgo"
__email__ = "dfidalgo@argotec.es"
__status__ = "Development"


class IEEE802154Server(Server):

    CONFIGURATION_PARAMS = ['controller_id', 'pan_id', 'channel']

    def __init__(self, id: str, configuration: dict):
        self.radio = None
        self.state = False
        Server.__init__(self, id, 'ieee802154.IEEE802154Server', configuration)

    def start(self):
        self.state = True
        self.radio.start()
        self.trigger_event('server_start', communicator=self, configuration=self.configuration)

    def treat_request(self, configuration: dict, device_id: str):
        client_address = configuration
        client_address['device_id'] = device_id
        radio_client = IEEE802154Client('netbows_rf_tmp_client', configuration)
        self.trigger_event('server_request', address=client_address, client=radio_client)

    def configure(self):
        self.radio = MRF24J40MRadio()
        self.radio.set_request_func(self.treat_request)
        self.radio.configure(self.configuration)

    def stop(self):
        message = 'Succesfully close'
        try:
            self.radio.stop()
        except Exception as err:
            message = str(err)
        self.trigger_event('server_stop', communicator=self, message=message)


class IEEE802154Client(Client):

    CONFIGURATION_PARAMS = ['controller_id', 'device_id', 'pan_id', 'channel']

    def __init__(self, id: str, configuration: dict):
        self.radio = None
        Client.__init__(self, id, 'ieee802154.IEEE802154Client', configuration)

    def configure(self):
        self.radio = MRF24J40MRadio()

    def receive(self):
        message = self.radio.read_message(self.configuration['device_id'])
        self.trigger_event('client_rcv', communicator=self, frame=message)
        return message

    def send(self, message):
        self.radio.send_message(message, self.configuration['device_id'])
        self.trigger_event('client_send', communicator=self, frame=message)

    def connect(self):
        self.trigger_event('client_connection', communicator=self, configuration=self.configuration)

    def close(self):
        self.trigger_event('client_close', communicator=self, message='Close successfully')
