import socketserver
import socket
import threading

from iotgate.core.model.communicators import Client
from iotgate.core.model.communicators import Server

__author__ = "diego"
__copyright__ = "Copyright 2016, Argotec SL"
__license__ = "?"
__version__ = "0.1.4-tfg"
__maintainer__ = "Diego Fidalgo"
__email__ = "dfidalgo@argotec.es"
__status__ = "Development"


class ThreadedTCPServer(socketserver.ThreadingMixIn, socketserver.TCPServer):
    # for preventing the "address already in use" error when restarting the daemon
    allow_reuse_address = True


class ThreadedTCPRequestHandler(socketserver.BaseRequestHandler):
    callback = None

    def handle(self):
        configuration = {
            "host": self.client_address[0],
            "port": self.client_address[1]
        }
        client = TCPClient("request_client_tmp", configuration, sock=self.request)
        self.callback("server_request", address=configuration, client=client)


class TCPServer(Server):

    CONFIGURATION_PARAMS = ["host", "port"]

    def __init__(self, id: str, configuration: dict):
        self.server = None
        self.server_thread = None
        Server.__init__(self, id, 'tcp.TCPServer', configuration)

    def configure(self):
        host, port = self.configuration['host'], int(self.configuration['port'])
        ThreadedTCPRequestHandler.callback = self.trigger_event
        self.server = ThreadedTCPServer((host, port), ThreadedTCPRequestHandler)

    def start(self):
        self.server_thread = threading.Thread(target=self.server.serve_forever)
        self.server_thread.setDaemon(True)
        self.server_thread.start()
        self.trigger_event("server_start", communicator=self, configuration=self.configuration)

    def add_request_callback(self, callback):
        ThreadedTCPRequestHandler.callback = self.trigger_event

    def stop(self):
        self.server.shutdown()
        self.server.server_close()
        self.trigger_event("server_stop", communicator=self, configuration=self.configuration)


class TCPClient(Client):

    CONFIGURATION_PARAMS = ["host", "port"]

    def __init__(self, id: str, configuration: dict, sock=None):
        Client.__init__(self, id, 'tcp.TCPClient', configuration)
        if sock:
            self.sock = sock
            self.state = "connected"
        else:
            self.server_address = (self.configuration["host"], int(self.configuration["port"]))
            self.state = "ready_for_configure"

    def connect(self):
        self.configure()
        self.sock.connect(self.server_address)
        self.trigger_event("client_connect", communicator=self, configuration=self.configuration)

    def configure(self):
        self.sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self.sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)

        self.state = "ready_for_connect"

    def send(self, message: str):
        self.state = "sending"
        try:
            self.sock.sendall(bytes(message, "utf-8"))
            self.trigger_event("client_send", communicator=self, frame=message)
        except socket.error as err:
            message = "Failed to send frame ( " + str(self.server_address[0]) + "):\nException: " + str(err)
            self.trigger_event("error", message=message)
        self.state = "connected"

    def receive(self) -> str:
        self.state = "receiving"
        response = self.sock.recv(2**10)
        self.trigger_event("client_rcv", communicator=self, frame=response)
        self.state = "connected"
        return response.decode('utf-8')

    def close(self):
        err = "Close successfully"
        try:
            self.sock.close()
        except Exception as err:
            err = str(err)
        self.trigger_event("client_close", communicator=self, message=err)
        self.state = "disconnected"
