import configparser

__author__ = "diego"
__copyright__ = "Copyright 2016, Argotec SL"
__license__ = "?"
__version__ = "0.1.4-tfg"
__maintainer__ = "Diego Fidalgo"
__email__ = "dfidalgo@argotec.es"
__status__ = "Development"


class Configuration:
    def __init__(self, configuration_path):
        self.config_path = configuration_path

    def open_ini_file(self):
        configuration_file = open(self.config_path)
        config = configparser.RawConfigParser(allow_no_value=True)
        config.read_file(configuration_file)
        configuration_file.close()
        return config

    def get_configuration_from_path(self):
        config = self.open_ini_file()
        configuration = {}

        for section in config.sections():
            configuration[section] = {}
            for option in config.options(section):
                configuration[section][option] = config.get(section, option)

        return configuration


class PersistenceConfiguration(Configuration):
    def __init__(self, configuration_path):
        Configuration.__init__(self, configuration_path)
        self.parsed_config = self.get_configuration_from_path()

    def get_engine_configuration(self):
        persistence_config = self.parsed_config['persistence']
        config = {'engine': None}
        if persistence_config['driver'] == 'sqlite':
            config['engine'] = 'sqlite:///' + persistence_config['database_file']
        elif persistence_config['driver'] in ('mysql', 'postgresql'):
            config['engine'] = persistence_config["driver"] + '://' +\
                               persistence_config['user'] + ":" +\
                               persistence_config['password'] +\
                               '@' + persistence_config['host'] +\
                               ':' + persistence_config['port'] +\
                               '/' + persistence_config['database_name']
        else:
            # TODO: Raise exception
            raise Exception('Database configuration err: unknown driver')

        return config


class LoggerConfiguration(Configuration):
    def __init__(self, configuration_path):
        Configuration.__init__(self, configuration_path)
        self.parsed_config = self.get_configuration_from_path()
        self.parsed_config = self.parsed_config['logger']

    def get_logger_configuration(self):
        try:
            self.parsed_config['syslog'] = True if 'True' == self.parsed_config['syslog'] else False
            self.parsed_config['verbose'] = True if 'True' == self.parsed_config['verbose'] else False
            self.parsed_config['own'] = True if 'True' == self.parsed_config['syslog'] else False
        except:
            # TODO: Raise exception
            raise Exception('Logger configuration err')

        return self.parsed_config


class DeploymentConfiguration(Configuration):
    def __init__(self, configuration_path):
        Configuration.__init__(self, configuration_path)
        self.parsed_config = self.get_configuration_from_path()
        self.parsed_config = self.parsed_config['deployment']

    def get_deployment_configuration(self):
        return self.parsed_config
