import syslog
import datetime
import inspect
import json
import _thread

__author__ = "diego"
__copyright__ = "Copyright 2016, Argotec SL"
__license__ = "?"
__version__ = "0.1.4-tfg"
__maintainer__ = "Diego Fidalgo"
__email__ = "dfidalgo@argotec.es"
__status__ = "Development"


class Logger(object):

    @staticmethod
    def get_callee_class(stack):
        the_class = stack[1][0].f_locals['self'].__class__
        # the_method = stack[1][0].f_code.co_name
        return the_class.__name__

    def __init__(self, logger_config):
        self.LOGGER_PATH = logger_config['log_path']
        self.VERBOSE = logger_config['verbose']
        self.USE_OWN_LOG = logger_config['own']
        self.USE_SYSLOG = logger_config['syslog']

    def info(self, message):
        thread_id = _thread.get_ident()
        if len(message.split(':::')) > 1:
            message = message.split(':::')[1]
            severity = message.split(':::')[0]
        else:
            severity = 'None'

        pline = '[Thread: {thread_id} | {callee_name} | {time} ] {message}'
        callee_name = inspect.currentframe().f_back.f_locals['self'].__class__.__name__
        line = pline.format(thread_id=thread_id, callee_name=callee_name, time=datetime.datetime.now(), message=message)
        if severity == "error":
            try:
                log = open(self.LOGGER_PATH + 'iotgate.err', 'a')
            except IOError:
                log = open(self.LOGGER_PATH + 'iotgate.err', 'w')
                log.close()
                log = open(self.LOGGER_PATH + 'iotgate.err', 'a')
            if self.USE_SYSLOG:
                syslog.syslog(syslog.LOG_ERR, line)
        else:
            try:
                log = open(self.LOGGER_PATH + 'iotgate.log', 'a')
            except IOError:
                log = open(self.LOGGER_PATH + 'iotgate.log', 'w')
                log.close()
                log = open(self.LOGGER_PATH + 'iotgate.log', 'a')
            if self.USE_SYSLOG:
                syslog.syslog(line)

        if self.USE_OWN_LOG:
            log.write(line + '\n')
            log.close()

        if self.VERBOSE:
            print(line)

    def log_adaptor_error(self, message):
        callee_name = self.get_callee_class(inspect.stack())

        self.info(message)

    def log_on_connection(self, is_known, device, device_address):
        callee_name = self.get_callee_class(inspect.stack())
        info = json.dumps(device_address.copy(), indent=3)
        pline = 'New connection from {unknown}{device}: {info}'
        line = pline.format(unknown='' if is_known else 'unknown ', device=device, info=info)

        self.info(line)
