from datetime import datetime
import random
import time

from iotgate.core.model.nodes import NodeUser
from iotgate.core.model.products import Product
from iotgate.core.model.products import ProductActuator
from iotgate.core.model.products import ProductActuatorAction
from iotgate.core.model.products import ProductSensor
from iotgate.deployment.serializer import ProductSerializer


def test_thermostat(engine):
    group = engine.node_manager.create_node_group("MainGroup", "Thermostat 32TFT Group")

    product = ProductSerializer.from_descriptor({
        "id": "Thermostat32TFT",
        "manufacturer": "Argotec",
        "model": "netbows",
        "components": [{
            "type": "actuator",
            "board_id": "*",
            "unit": "unspecified",
            "value_type": "str",
            "actions": [{"name": "reset", "command": "at!comr", "effect": ""},
                        {"name": "sendData", "command": "at!cosd10", "effect": ""},
                        {"name": "setTemp", "command": "at!coct{temp}", "effect": ""},
                        {"name": "on", "command": "at!on", "effect": ""},
                        {"name": "off", "command": "at!ac022on", "effect": ""}]
        }, {
            "type": "sensor",
            "board_id": "temperature",
            "unit": "celsius",
            "value_type": "float",
        }, {
            "type": "sensor",
            "board_id": "humidity",
            "unit": "percentage",
            "value_type": "float",
        }]
    })
    engine.product_manager.register_product(product)

    comm = engine.communicator_manager.new_communicator("RFServer", "ieee802154.IEEE80154Server", {
        "controller_id": "b000",
        "pan_id": "2345",
        "channel": "15"
    })
    adaptor = engine.adaptor_manager.new_adaptor("RFAdaptor", "argotec.smarthome.Thermostat", [comm])

    return engine.node_manager.create_node("Thermostat32TFT-1", "2", group, product, adaptor)
